/*
 *    Copyright 1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
#include <stdio.h>
#include <X11/Xos.h>
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/Frame.h>
#include <Xm/BulletinB.h>
#include "xmap.h"
#include "gps.h"
#include "debug.h"

extern Widget draww;

static int objvisible = 0;
static int laststat = 0;
static double lastlat = -1000.0;
static double lastlon = -1000.0;
extern struct map map;
void do_track(void);

static int off_edge(Position ax, Position ay, double lat, double lon)
{
  int dir = 0;

  if (ax < 0)
    dir+=8;
  else
    if (ax > map.mwidth)
      dir+=2;
  if (ay < 0)
    dir+=1;
  else
    if (ay > map.mheight)
      dir+=4;
  if (dir)
    return dir;

  if (map.wedge && (lon < map.wedge))
    dir+=8;
  else
    if (map.eedge && (lon > map.eedge))
      dir+=2;
  if (map.nedge && (lat > map.nedge))
    dir+=1;
  else
    if (map.sedge && (lat < map.sedge))
      dir+=4;
  return dir;
}

void load_map(char *newmap)
{
  int testzoom, i, lastzoom;
  char bufp[128];
  lastzoom = mapdata.zoom;
  if (newmap && strcmp(newmap, "")) {
      strncpy(bufp, newmap, 128);
      i = strlen(bufp);
      strcpy(bufp + (i-4), ".ini");
      new_map(bufp);
      while (mapdata.zoom < lastzoom)
      {
        testzoom = mapdata.zoom;
        zoomout(NULL, &mapdata);
        if (testzoom == mapdata.zoom)
          break;
      }
      while (mapdata.zoom > lastzoom)
      {
        testzoom = mapdata.zoom;
        zoomin(NULL, &mapdata);
        if (testzoom == mapdata.zoom)
          break;
      }
      do_track();
  }
}

void mark_track(void)
{
    int dir;
    Position x, y, ax, ay;
    Dimension width, height;

    if (gpsdata.status > 0) {
        // Update this early.
	latlontoxy(gpsdata.latitude, gpsdata.longitude, &x, &y);
	XtVaGetValues(draww,
		XmNwidth, &width,
		XmNheight, &height,
		NULL);
	//
	// Keep the target centered, depending on center mode
	//
	if ( x<20 || y<20 || x>(width-20) || y>(height-20)) {
	    center(x, y);
	    latlontoxy(gpsdata.latitude, gpsdata.longitude, &x, &y);
	}

      ax = x * mapdata.zoom + mapdata.mx_offset;
      ay = y * mapdata.zoom + mapdata.my_offset;

      dir = off_edge(ax, ay, gpsdata.latitude, gpsdata.longitude);
      if (dir != 0)
        switch (dir) 
          {
          case 1:
            load_map(map.nmap);
            break;
          case 3:
            load_map(map.nemap);
            break;
          case 2:
            load_map(map.emap);
            break;
          case 6:
            load_map(map.semap);
            break;
          case 4:
            load_map(map.smap);
            break;
          case 12:
            load_map(map.swmap);
            break;
          case 8:
            load_map(map.wmap);
            break;
          case 9:
            load_map(map.nwmap);
            break;
          }
      else
        {
          graph_mark(gpsdata.mode?gpsdata.mode:3, x, y);

          lastlat = gpsdata.latitude;
          lastlon = gpsdata.longitude;
          laststat = gpsdata.status;
          objvisible = 1;
        }
    }
}


void redraw_mark(Widget w, void *client_data, void *cbs)
{
    Position x, y;

    if (objvisible) {
	latlontoxy(lastlat, lastlon, &x, &y);
	graph_mark(gpsdata.mode?gpsdata.mode:gpsdata.status?3:1, x, y);
    }
}




void unmark_track(void)
{
    Position x, y;

    if (objvisible) {
	latlontoxy(lastlat, lastlon, &x, &y);
	graph_unmark(x, y);
	objvisible = 0;
    }
}

void track_start(void)
{
    if (gpsdata.status == -1) {
	gps_connect();
	lastlat = lastlon = -1000.0;
	objvisible = 0;
    } else
	beep();
}

void track_stop(void)
{
    if (gpsdata.status != -1) {
	// This also implies to stop loging.
	close_log();
	gps_unconnect();
	if (lastlat != -1000.0) {
	    unmark_track();
	}
    } else
	beep();
}

void track_toggle(void)
{
    if (gpsdata.status == -1)
	track_start();
    else
	track_stop();
}

void do_track(void)
{
    if (lastlat != gpsdata.latitude
	|| lastlon != gpsdata.longitude
	|| laststat != gpsdata.status) {
	unmark_track();
	mark_track();
    }
}
